//+------------------------------------------------------------------+
//|                                                     SignalWZ.mqh |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
#include <My\RBM_r4--.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals based on unconventional rbm                        |
//| Type=SignalAdvanced                                              |
//| Name=RBM                                                         |
//| ShortName=RBM                                                    |
//| Class=CSignalRBM                                                 |
//| Page=signal_kmeans                                               |
//| Parameter=Sample,int,100,Sample                                  |
//| Parameter=Loss,double,0.9,Marker Threshold                       |
//| Parameter=File,string,"rbm",File Name Prefix                     |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalRBM.                                                |
//| Purpose: Class of of trade signals based on unconventional rbm   | 
//| Boltzmann Machine NN.                                            |
//|            Derives from class CExpertSignal.                     |
//+------------------------------------------------------------------+
#define __VISIBLE 5
#define __HIDDEN 3
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalRBM                       : public CExpertSignal
  {
protected:
   
   CiMA                                MA;
   CiMACD                              MACD;
   CiWPR                               WPR;
   CiStochastic                        STOCH;
   CiRVI                               RVI;
   
   int                                 m_sample;
   double                              m_loss;
   string                              m_file;
   
   bool                                m_load;

   Crbm                                *RBM;
   double                              GetOutput();

public:
   
   //--- method of verification of settings
   virtual bool                        ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool                        InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int                         LongCondition(void);
   virtual int                         ShortCondition(void);
   //---
   void                                Sample(int value){ m_sample=value;     }
   void                                Loss(double value){ m_loss=value;}
   void                                File(string value){ m_file=value;            }
   
                                       CSignalRBM(void);
                                       ~CSignalRBM(void);          

};  
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CSignalRBM::CSignalRBM(void)  :   m_sample(100),
                                       m_loss(0.9),
                                       m_file("rbm")
  {
//--- initialization of protected data
      m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE+USE_SERIES_SPREAD+USE_SERIES_TIME;
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalRBM::ValidationSettings(void)
  {
   if(!CExpertSignal::ValidationSettings())
      return(false);
//--- initial data checks

   if(m_sample<=0)
   {
      printf(__FUNCTION__+": iterations should exceed 0. ");
      return(false);
   }
   
      RBM = new Crbm(5,__HIDDEN,m_sample,m_loss,m_file);
   
      RBM.ReadWeights();
      
      m_load=false;
   
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalRBM::InitIndicators(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
   
   if(!MA.Create(m_symbol.Name(),m_period,__VISIBLE,0,MODE_SMA,PRICE_CLOSE))
      return(false);

   if(!MACD.Create(m_symbol.Name(),m_period,12,26,9,PRICE_CLOSE))
      return(false);

   if(!WPR.Create(m_symbol.Name(),m_period,__VISIBLE))
      return(false);

   if(!STOCH.Create(m_symbol.Name(),m_period,__VISIBLE,3,3,MODE_SMA,STO_LOWHIGH))
      return(false);

   if(!RVI.Create(m_symbol.Name(),m_period,__VISIBLE))
      return(false);

//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalRBM::LongCondition(void)
  {
      int _result=0;
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _i=StartIndex();
      
      double _o=GetOutput();
      
      //normalize point move to within 100
      if(_o>0.0){ _result=int(fmin(100.0,100.0*(_o/(m_high.GetData(_i)-m_low.GetData(_i))))); }
   
      return(_result);
  }
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalRBM::ShortCondition(void)
  {
      int _result=0;
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _i=StartIndex();
      
      double _o=GetOutput();
      
      //normalize point move to within 100
      if(_o<0.0){ _result=int(fmin(100.0,100.0*(fabs(_o)/(m_high.GetData(_i)-m_low.GetData(_i))))); }
   
      return(_result);
  }
//+------------------------------------------------------------------+
//| RBM Output.                                                      |
//+------------------------------------------------------------------+
double CSignalRBM::GetOutput(void)
   {
      m_close.Refresh(-1);
      
      MA.Refresh(-1);
      MACD.Refresh(-1);
      STOCH.Refresh(-1);
      WPR.Refresh(-1);
      RVI.Refresh(-1);
      
      double _output = 0.0;
      
      int _i=StartIndex();
      
      matrix _w; _w.Init(m_sample,__VISIBLE);
      
      matrix _xy; _xy.Init(m_sample,7);
      
      if(RBM.init)
      {
         for(int s=0;s<m_sample;s++)
         {
            for(int i=0;i<5;i++)
            {
               if(i==0){ _w[s][i] = MA.GetData(0,_i+s); }
               else if(i==1){ _w[s][i] = MACD.GetData(0,_i+s); }
               else if(i==2){ _w[s][i] = WPR.GetData(0,_i+s); }
               else if(i==3){ _w[s][i] = STOCH.GetData(0,_i+s); }
               else if(i==4){ _w[s][i] = RVI.GetData(0,_i+s); }
            }
            
            if(s>0){ _xy[s][2*__HIDDEN] = m_close.GetData(_i+s)-m_close.GetData(_i+s+1); }

         }
         
         RBM.Train(_w);
         RBM.Process(_w,_xy);
         
         double _w=0.0,_w_sum=0.0;
         
         vector _x0=_xy.Row(0); _x0.Resize(6);
         
         for(int s=1;s<m_sample;s++)
         {
            vector _x=_xy.Row(s); _x.Resize(6);
            double _weight=fabs(1.0+_x.Loss(_x0,LOSS_COSINE));
            
            _w+=(_weight*_xy[s][6]);
            _w_sum+=_weight;
         }
         
         if(_w_sum>0.0){ _w/=_w_sum; }
         
         _output=_w;
      }
      
      return(_output);
   }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
void CSignalRBM::~CSignalRBM(void)
  {
   RBM.WriteWeights();
   delete RBM;
  }
//+------------------------------------------------------------------+
